<?php

namespace Drupal\persistent_visitor_parameters\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\persistent_visitor_parameters\PersistorCookie;

/**
 * The persistent_visitor_parameters settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames() {
    return [
      'persistent_visitor_parameters.settings',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId() {
    return 'persistent_visitor_parameters_settings_form';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('persistent_visitor_parameters.settings');

    $getParametersDefaultValue = implode('|', $config->get('get_parameters'));

    $form['get_parameters'] = [
      '#type' => 'textfield',
      '#title' => $this->t('List of GET parameters'),
      '#description' => $this->t('List GET parameters for tracking, separated by "|". Example: "utm_source|utm_medium|utm_campaign|utm_content|utm_term". Leave empty to disable GET parameter tracking and persistence.'),
      '#default_value' => $getParametersDefaultValue,
    ];

    $serverParametersDefaultValue = implode('|', $config->get('server_parameters'));

    $form['server_parameters'] = [
      '#type' => 'textfield',
      '#title' => $this->t('List of SERVER parameters'),
      '#description' => $this->t('List of SERVER parameters for tracking, separated by "|". Example: "HTTP_REFERER|HTTP_USER_AGENT". Leave empty to disable SERVER parameter tracking and persistence.'),
      '#default_value' => $serverParametersDefaultValue,
    ];

    $form['server_parameters_on_get_parameters_only'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Only track server parameters if at least one of the get parameters is present.'),
      '#default_value' => $config->get('server_parameters_on_get_parameters_only'),
      '#states' => [
        'visible' => [
          ':input[name="get_parameters"]' => ['!value' => ''],
          'and' => 'and',
          ':input[name="server_parameters"]' => ['!value' => ''],
        ],
        'unchecked' => [
          [':input[name="get_parameters"]' => ['value' => '']],
          [':input[name="server_parameters"]' => ['value' => '']],
        ],
      ],
    ];

    $form['mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Parameters save mode'),
      '#description' => $this->t('Consider only very first-touch parameters, or allow it to be overridden with last-touch parameters?'),
      '#options' => [
        'first_touch' => $this->t('First-touch'),
        'last_touch' => $this->t('Last-touch'),
      ],
      '#required' => TRUE,
      '#default_value' => $config->get('mode'),
    ];

    $form['persistor'] = [
      '#type' => 'radios',
      '#title' => $this->t('Select persistence storage method'),
      '#description' => nl2br($this->t("Select the store where you want to persist the data. Each store has pro's and cons:\n
        <strong>Private Temp Store:</strong> @todo\n
        <strong>Session:</strong> @todo\n
        <strong>Cookie:</strong> @todo\n
        <strong>Disabled:</strong> @todo\n")),
      '#options' => [
        'persistor_private_temp_store' => $this->t('Private Temp Store'),
        'persistor_session' => $this->t('Session'),
        'persistor_cookie' => $this->t('Cookie'),
        'persistor_null' => $this->t('Disabled: Do not store persistently'),
      ],
      '#required' => TRUE,
      '#default_value' => $config->get('persistor'),
    ];

    $form['persistor_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Persistence key / Cookie name'),
      '#description' => $this->t('The key to use for storing the parameters persistently, based on the selected persistence method, e.g. Session value key, Private Temp Store key, cookie name. Warning: Changing this key will reset persistent values stored.'),
      '#default_value' => $config->get('persistor_key'),
      '#required' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="persistor"]' => ['!value' => 'persistor_null'],
        ],
      ],
    ];

    $cookieExpireDefaultValue = $config->get('persistor_expire');
    if ($cookieExpireDefaultValue != PersistorCookie::COOKIE_EXPIRE_SESSION && $cookieExpireDefaultValue != PersistorCookie::COOKIE_EXPIRE_NEVER) {
      $cookieExpireDefaultValue = 'custom';
    }
    $form['persistor_expire'] = [
      '#type' => 'radios',
      '#title' => $this->t('Cookie expiration'),
      '#options' => [
        PersistorCookie::COOKIE_EXPIRE_SESSION => $this->t('Session'),
        PersistorCookie::COOKIE_EXPIRE_NEVER => $this->t('Never'),
        'custom' => $this->t('Custom'),
      ],
      '#required' => TRUE,
      '#default_value' => $cookieExpireDefaultValue,
      '#states' => [
        'visible' => [
          ':input[name="persistor"]' => ['value' => 'persistor_cookie'],
        ],
      ],
    ];
    $form['persistor_expire_custom'] = [
      '#type' => 'number',
      '#title' => $this->t('Custom duration'),
      '#description' => $this->t('The time the cookie expires. This is the number of seconds from the current time.'),
      '#default_value' => $config->get('persistor_expire'),
      '#states' => [
        'visible' => [
          ':input[name="persistor_expire"]' => ['value' => 'custom'],
          'and' => 'and',
          ':input[name="persistor"]' => ['value' => 'persistor_cookie'],
        ],
        'required' => [
          ':input[name="persistor_expire"]' => ['value' => 'custom'],
          'and' => 'and',
          ':input[name="persistor"]' => ['value' => 'persistor_cookie'],
        ],
      ],
    ];

    $form['respect_dnt'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Respect DNT?"),
      '#description' => $this->t('Respect DNT (Do Not Track) and do not persist the user data if true.'),
      '#default_value' => $config->get('respect_dnt'),
    ];

    $form['allow_unsafe_deserialization'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Allow unsafe deserialization [LEGACY]'),
      '#description' => $this->t('Cookie content used to be PHP-serialized instead of JSON encoded. Enable it, to allow unserializing cookie content if it is not valid json (legacy fallback). Be careful with this option, as it may lead to security vulnerabilities. Support for serialization will be removed in the next major release (2.x).'),
      '#default_value' => $config->get('allow_unsafe_deserialization'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $cookieExpireValue = $form_state->getValue('persistor_expire');
    if ($cookieExpireValue == 'custom') {
      $cookieExpireValue = $form_state->getValue('persistor_expire_custom');
    }

    $this->config('persistent_visitor_parameters.settings')
      ->set('get_parameters', !empty($form_state->getValue('get_parameters')) ? array_filter(array_map('trim', explode('|', $form_state->getValue('get_parameters')))) : [])
      ->set('server_parameters', !empty($form_state->getValue('server_parameters')) ? array_filter(array_map('trim', explode('|', $form_state->getValue('server_parameters')))) : [])
      ->set('mode', $form_state->getValue('mode'))
      ->set('persistor', $form_state->getValue('persistor'))
      ->set('persistor_key', $form_state->getValue('persistor_key'))
      ->set('persistor_expire', $cookieExpireValue)
      ->set('respect_dnt', $form_state->getValue('respect_dnt'))
      ->set('server_parameters_on_get_parameters_only', $form_state->getValue('server_parameters_on_get_parameters_only'))
      ->set('allow_unsafe_deserialization', $form_state->getValue('allow_unsafe_deserialization'))
      ->save();
  }

}
