<?php

declare(strict_types=1);

namespace Drupal\persistent_visitor_parameters;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The parameter get manager.
 *
 * Handles the parameters from GET / SERVER.
 */
final class ParameterManager {

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * Constructs a ParameterManager object.
   */
  public function __construct(
    protected readonly RequestStack $requestStack,
    protected readonly ConfigFactoryInterface $configFactory,
  ) {
    $this->config = $configFactory->get('persistent_visitor_parameters.settings');
  }

  /**
   * Returns true if the current request has DNT header and respect_dnt active.
   *
   * @return bool
   *   Should not be tracked, as DNT is active!
   */
  public function dntApplies(): bool {
    // Respect Do Not Track if not otherwise configured.
    return $this->requestStack->getCurrentRequest()->server->get('HTTP_DNT') && $this->config->get('respect_dnt');
  }

  /**
   * Gets the selected GET parameters from the request.
   *
   * @return array
   *   An array of selected GET parameters.
   */
  public function getMatchingGetParameters(): array {
    $query = $this->requestStack->getCurrentRequest()->query->all();
    $queryParams = UrlHelper::filterQueryParameters($query);
    return array_intersect_key($queryParams ?? [], array_flip($this->config->get('get_parameters') ?? []));
  }

  /**
   * Gets the selected server parameters from the request.
   *
   * @return array
   *   An array of selected server parameters.
   */
  public function getMatchingServerParameters(): array {
    $serverParams = $this->requestStack->getCurrentRequest()->server->all();
    return array_intersect_key($serverParams ?? [], array_flip($this->config->get('server_parameters') ?? []));
  }

}
