<?php

namespace Drupal\persistent_visitor_parameters;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Render\HtmlResponse;

/**
 * The cookie manager service.
 */
class PersistenceManager {

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * Constructs a CookieManager object.
   *
   * @param ParameterManager $parameterManager
   *   The parameter manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    protected ParameterManager $parameterManager,
    ConfigFactoryInterface $configFactory,
  ) {
    $this->config = $configFactory->get('persistent_visitor_parameters.settings');
  }

  /**
   * Helper method to return the parameters data array.
   *
   * @return array|null
   *   The current request parameters array
   */
  protected function getMatchingRequestParameters(): ?array {
    // If this request has DNT enabled and we respect DNT, return early:
    if ($this->parameterManager->dntApplies()) {
      return NULL;
    }

    $getParameters = $this->parameterManager->getMatchingGetParameters();
    $serverParameters = $this->parameterManager->getMatchingServerParameters();

    // If we are tracking with GET parameters only, and there are no GET
    // parameters, we can return early:
    if ($this->config->get('server_parameters_on_get_parameters_only') && empty($getParameters)) {
      return NULL;
    }
    $data = array_merge($getParameters, $serverParameters);

    // If expected parameters are missing, we return NULL:
    if (empty($data)) {
      return NULL;
    }
    return $data;
  }

  /**
   * Stores the current request parameters, if matching.
   *
   * @param \Drupal\Core\Render\HtmlResponse $response
   *   The current request response.
   *
   * @return bool
   *   TRUE if parameters were persisted, else FALSE.
   */
  public function persistMatchingRequestParameters(HtmlResponse $response): bool {
    $data = $this->getMatchingRequestParameters();

    // If data is empty, we can return early:
    if (empty($data)) {
      return FALSE;
    }

    // If first-touch mode activated, and cookie already exists... skip the new
    // one. Otherwise - replace it with a new one:
    if ($this->config->get('mode') === 'first_touch' && !empty($this->loadPersistedParameters($response))) {
      return FALSE;
    }

    return $this->getPersistor()->persist($data, $response);
  }

  /**
   * Retrieves the cookie content.
   *
   * @return array|null
   *   The decoded cookie content or NULL if the cookie does not exist.
   */
  public function loadPersistedParameters(): ?array {
    return $this->getPersistor()->load();
  }

  /**
   * Returns the persistor selected in config.
   *
   * @return PersistorInterface
   *   The persistor to use.
   */
  public function getPersistor(): PersistorInterface {
    $selectedPersistorService = $this->config->get('persistor');
    // We dynamically load the service here, DI doesn't make much sense:
    // phpcs:disable
    // @phpstan-ignore-next-line
    $persistor = \Drupal::service("persistent_visitor_parameters.{$selectedPersistorService}");
    // phpcs:enable
    return $persistor;
  }

}
