<?php

declare(strict_types=1);

namespace Drupal\persistent_visitor_parameters;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Render\HtmlResponse;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * The cookie persistor class.
 */
class PersistorCookie extends PersistorBase implements PersistorInterface {

  const COOKIE_EXPIRE_SESSION = 0;
  const COOKIE_EXPIRE_NEVER = 2147483647;

  /**
   * The constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factors.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack.
   * @param \Drupal\Component\Datetime\TimeInterface $datetimeTime
   *   The TimeInterface.
   */
  public function __construct(
    protected readonly ConfigFactoryInterface $configFactory,
    protected readonly RequestStack $requestStack,
    protected readonly TimeInterface $datetimeTime,
  ) {
    $this->config = $configFactory->get('persistent_visitor_parameters.settings');
  }

  /**
   * {@inheritDoc}
   */
  public function clear(): bool {
    $this->requestStack->getCurrentRequest()->cookies->remove($this->getPersistorKey());
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function persist(array $data, ?HtmlResponse $response): bool {
    $cookieExpire = $this->config->get('persistor_expire');

    // If we are using custom cookie expiration, we need to add the current time
    // to the expiration time, when setting the cookie:
    if ($cookieExpire != self::COOKIE_EXPIRE_SESSION && $cookieExpire != self::COOKIE_EXPIRE_NEVER) {
      $cookieExpire += $this->datetimeTime->getRequestTime();
    }

    $persistorKey = $this->getPersistorKey();
    $cookie = Cookie::create($persistorKey, Json::encode($data), $cookieExpire);
    $response->headers->setCookie($cookie);

    return !empty($response->headers->getCookies()[$persistorKey]);
  }

  /**
   * {@inheritdoc}
   */
  public function load(): ?array {
    $cookie = $this->requestStack->getCurrentRequest()->cookies->get($this->getPersistorKey());
    if (empty($cookie)) {
      return NULL;
    }
    $decodedCookie = Json::decode($cookie);

    // @deprecated 2.1.0:
    // @todo Remove this in next major release, no deprecation notice needed
    // here, as it would spam the logs.
    // If the decoding failed, $decodedCookie will be NULL or FALSE, so we try
    // to unserialize the cookie content if the configuration allows it:
    if ($this->config->get('allow_unsafe_deserialization') && empty($decodedCookie)) {
      // @codingStandardsIgnoreStart
      $decodedCookie = unserialize($cookie);
      // @codingStandardsIgnoreEnd
    }
    return $decodedCookie;
  }

}
