<?php

namespace Drupal\Tests\persistent_visitor_parameters\Functional;

use Drupal\persistent_visitor_parameters\ParameterManager;
use Drupal\persistent_visitor_parameters\PersistenceManager;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;

/**
 * This class provides methods for testing persistent_visitor_parameters.
 *
 * @group persistent_visitor_parameters
 */
abstract class PersistorTestBase extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'test_page_test',
    'persistent_visitor_parameters',
  ];

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The persistence manager service.
   *
   * @var \Drupal\persistent_visitor_parameters\PersistenceManager
   */
  protected ?PersistenceManager $persistenceManager;

  /**
   * The parameter manager service.
   *
   * @var \Drupal\persistent_visitor_parameters\ParameterManager|null
   */
  protected ?ParameterManager $parameterManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->config('system.site')->set('page.front', '/test-page')->save();
    // Give access persistent_visitor_parameters debug permission
    // so that we can access the debug page.
    // This is needed, because we don't know how to easily get the
    // users session / cookie / ... values as the test runner is
    // in a different process and we can't simply call
    // $this->persistenceManager->loadPersistedParameters();
    $this->grantPermissions(Role::load(RoleInterface::ANONYMOUS_ID), ['access persistent_visitor_parameters debug']);
    $this->persistenceManager = \Drupal::service('persistent_visitor_parameters.persistence_manager');

    // Set the configuration programmatically, so we don't have to make any
    // requests to the settings form:
    $this->config('persistent_visitor_parameters.settings')
      ->set('get_parameters', ['utm_source', 'utm_medium', 'utm_campaign', 'utm_term', 'utm_content'])
      ->set('server_parameters', ['HTTP_REFERER'])
      ->set('mode', 'last_touch')
      ->set('respect_dnt', FALSE)
      ->save();
  }

  /**
   * Helper method to return example UTM get parameters.
   */
  protected function getGetParameters1(): array {
    return [
      'utm_source' => 'foo',
      'utm_medium' => 'bar',
      'utm_campaign' => 'baz',
      'utm_term' => 'qux',
      'utm_content' => 'tak',
    ];
  }

  /**
   * Helper method to return different example UTM get parameters.
   */
  protected function getGetParameters2(): array {
    return [
      'utm_source' => 'foo1',
      'utm_medium' => 'bar1',
      'utm_campaign' => 'baz1',
      'utm_term' => 'qux1',
      'utm_content' => 'tak1',
    ];
  }

  /**
   * Tests persisting as authenticated user with first_touch method.
   */
  public function testPersistAuthenticatedFirstTouch() {
    // We don't support tracking for logged in users yet:
    $this->drupalGet('<front>', [
      'query' => $this->getGetParameters1(),
    ]);

    // @improve: We currently test against the debug page
    // because the test runner has a different session than the
    // actually tested user.
    // So we can't simply run
    // $this->persistenceManager->loadPersistedParameters();
    // in the tests, as it would always return NULL.
    // Maybe it's possible to instead use a mock object that is then filled
    // by the test or whatever, but after investing several hours looking
    // for solutions and documentation we gave up...
    // This works good enough for now. Please teach us! ;)
    $this->drupalGet('/admin/config/people/persistent-visitor-parameters/debug');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Persistent Visitor Parameters Debug');
    $this->assertSession()->pageTextContains('Array');
    $this->assertSession()->pageTextContains('[utm_source] => foo');
    $this->assertSession()->pageTextContains('[utm_medium] => bar');
    $this->assertSession()->pageTextContains('[utm_campaign] => baz');
    $this->assertSession()->pageTextContains('[utm_term] => qux');
    $this->assertSession()->pageTextContains('[utm_content] => tak');
    $this->assertSession()->pageTextContains('[HTTP_REFERER]');

    // @todo Add, once we have authenticated users support:
    // Make a second call:
    // $this->drupalGet('<front>', [
    // 'query' => $this->getGetParameters2(),
    // ]);
  }

  /**
   * Tests persisting as anonymous user with first_touch method.
   */
  public function testPersistAnonymousFirstTouch() {
    // Make a first call:
    $this->drupalGet('<front>', [
      'query' => $this->getGetParameters1(),
    ]);
    // @improve: We currently test against the debug page
    // because the test runner has a different session than the
    // actually tested user.
    // So we can't simply run
    // $this->persistenceManager->loadPersistedParameters();
    // in the tests, as it would always return NULL.
    // Maybe it's possible to instead use a mock object that is then filled
    // by the test or whatever, but after investing several hours looking
    // for solutions and documentation we gave up...
    // This works good enough for now. Please teach us! ;)
    $this->drupalGet('/admin/config/people/persistent-visitor-parameters/debug');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Persistent Visitor Parameters Debug');
    $this->assertSession()->pageTextContains('Array');
    $this->assertSession()->pageTextContains('[utm_source] => foo');
    $this->assertSession()->pageTextContains('[utm_medium] => bar');
    $this->assertSession()->pageTextContains('[utm_campaign] => baz');
    $this->assertSession()->pageTextContains('[utm_term] => qux');
    $this->assertSession()->pageTextContains('[utm_content] => tak');
    $this->assertSession()->pageTextContains('[HTTP_REFERER]');

    // Make a second call:
    $this->drupalGet('<front>', [
      'query' => $this->getGetParameters2(),
    ]);
    // Results should be same to first call:
    // @improve: We currently test against the debug page
    // because the test runner has a different session than the
    // actually tested user.
    // So we can't simply run
    // $this->persistenceManager->loadPersistedParameters();
    // in the tests, as it would always return NULL.
    // Maybe it's possible to instead use a mock object that is then filled
    // by the test or whatever, but after investing several hours looking
    // for solutions and documentation we gave up...
    // This works good enough for now. Please teach us! ;)
    $this->drupalGet('/admin/config/people/persistent-visitor-parameters/debug');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Persistent Visitor Parameters Debug');
    $this->assertSession()->pageTextContains('Array');
    $this->assertSession()->pageTextContains('[utm_source] => foo');
    $this->assertSession()->pageTextContains('[utm_medium] => bar');
    $this->assertSession()->pageTextContains('[utm_campaign] => baz');
    $this->assertSession()->pageTextContains('[utm_term] => qux');
    $this->assertSession()->pageTextContains('[utm_content] => tak');
    $this->assertSession()->pageTextContains('[HTTP_REFERER]');
  }

  /**
   * Tests persisting as authenticated user with last_touch method.
   */
  protected function testPersistAuthenticatedLastTouch() {
    // We don't support tracking for logged in users yet:
    $this->drupalGet('<front>', [
      'query' => $this->getGetParameters1(),
    ]);
    // @improve: We currently test against the debug page
    // because the test runner has a different session than the
    // actually tested user.
    // So we can't simply run
    // $this->persistenceManager->loadPersistedParameters();
    // in the tests, as it would always return NULL.
    // Maybe it's possible to instead use a mock object that is then filled
    // by the test or whatever, but after investing several hours looking
    // for solutions and documentation we gave up...
    // This works good enough for now. Please teach us! ;)
    $this->drupalGet('/admin/config/people/persistent-visitor-parameters/debug');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Persistent Visitor Parameters Debug');
    $this->assertSession()->pageTextContains('Array');
    $this->assertSession()->pageTextContains('[utm_source] => foo');
    $this->assertSession()->pageTextContains('[utm_medium] => bar');
    $this->assertSession()->pageTextContains('[utm_campaign] => baz');
    $this->assertSession()->pageTextContains('[utm_term] => qux');
    $this->assertSession()->pageTextContains('[utm_content] => tak');
    $this->assertSession()->pageTextContains('[HTTP_REFERER]');

    // @todo Add, once we have authenticated users support:
    // // Make a second call:
    // $this->drupalGet('<front>', [
    // 'query' => $this->getGetParameters2(),
    // ]);
  }

  /**
   * Tests persisting as anonymous user with last_touch method.
   */
  protected function testPersistAnonymousLastTouch() {
    // Make a first call:
    $this->drupalGet('<front>', [
      'query' => $this->getGetParameters1(),
    ]);
    // Expect the parameters:
    // @improve: We currently test against the debug page
    // because the test runner has a different session than the
    // actually tested user.
    // So we can't simply run
    // $this->persistenceManager->loadPersistedParameters();
    // in the tests, as it would always return NULL.
    // Maybe it's possible to instead use a mock object that is then filled
    // by the test or whatever, but after investing several hours looking
    // for solutions and documentation we gave up...
    // This works good enough for now. Please teach us! ;)
    $this->drupalGet('/admin/config/people/persistent-visitor-parameters/debug');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Persistent Visitor Parameters Debug');
    $this->assertSession()->pageTextContains('Array');
    $this->assertSession()->pageTextContains('[utm_source] => foo');
    $this->assertSession()->pageTextContains('[utm_medium] => bar');
    $this->assertSession()->pageTextContains('[utm_campaign] => baz');
    $this->assertSession()->pageTextContains('[utm_term] => qux');
    $this->assertSession()->pageTextContains('[utm_content] => tak');
    $this->assertSession()->pageTextContains('[HTTP_REFERER]');

    // Make a second call:
    $this->drupalGet('<front>', [
      'query' => $this->getGetParameters2(),
    ]);
    // Results should be different from first call:
    // @improve: We currently test against the debug page
    // because the test runner has a different session than the
    // actually tested user.
    // So we can't simply run
    // $this->persistenceManager->loadPersistedParameters();
    // in the tests, as it would always return NULL.
    // Maybe it's possible to instead use a mock object that is then filled
    // by the test or whatever, but after investing several hours looking
    // for solutions and documentation we gave up...
    // This works good enough for now. Please teach us! ;)
    $this->drupalGet('/admin/config/people/persistent-visitor-parameters/debug');
    $this->assertSession()->statusCodeEquals(200);
    $this->assertSession()->pageTextContains('Persistent Visitor Parameters Debug');
    $this->assertSession()->pageTextContains('Array');
    $this->assertSession()->pageTextContains('[utm_source] => foo1');
    $this->assertSession()->pageTextContains('[utm_medium] => bar1');
    $this->assertSession()->pageTextContains('[utm_campaign] => baz1');
    $this->assertSession()->pageTextContains('[utm_term] => qux1');
    $this->assertSession()->pageTextContains('[utm_content] => tak1');
    $this->assertSession()->pageTextContains('[HTTP_REFERER]');

  }

}
